#ifdef GL_ES
#define LOWP lowp
precision mediump float;
#else
#define LOWP
#endif

uniform sampler2D u_texture;
uniform LOWP vec3 u_tintRGB;
uniform float u_tintAmount;
uniform float u_saturation;
uniform float u_willInvert;
uniform float u_hueAdjust;

varying vec2 v_texCoord;
varying LOWP vec4 v_color;

const mat3 rgb2yiq = mat3(0.299, 0.595716, 0.211456, 0.587, -0.274453, -0.522591, 0.114, -0.321263, 0.311135);
const mat3 yiq2rgb = mat3(1.0, 1.0, 1.0, 0.9563, -0.2721, -1.1070, 0.6210, -0.6474, 1.7046);

void main()
{
	vec4 outColor = texture2D(u_texture, v_texCoord) * v_color;
	float alpha = outColor.a;

	if (alpha > 0.0) {
		// Hue shift
		if (u_hueAdjust > 0.0 && u_hueAdjust < 1.0) {
			vec3 yColor = rgb2yiq * outColor.rgb;

			//float originalHue = atan(yColor.b, yColor.g);
			// So atan(x,y) doesn't play nice if x==0.0, somehow it is for black/white and generally on IOS it didn't work - read the functions for "gotchas" if future glsl issues
			float originalHue = (yColor.g == 0.0 ? sign(yColor.b) * 1.570795 : atan(yColor.b, yColor.g));
			float finalHue = originalHue + u_hueAdjust * 6.28318;

			float chroma = sqrt(yColor.b * yColor.b + yColor.g * yColor.g);

			vec3 yFinalColor = vec3(yColor.r, chroma * cos(finalHue), chroma * sin(finalHue));
			outColor.rgb = yiq2rgb * yFinalColor;
		}

		// Tint
		outColor.rgb = mix(outColor.rgb, u_tintRGB * alpha, u_tintAmount);

		// Saturate
		vec3 w = vec3(0.2125, 0.7154, 0.0721);
		vec3 intensity = vec3(dot(outColor.rgb, w));
		outColor.rgb = mix(outColor.rgb, intensity, u_saturation + 1.0);
	}

    // Invert when setting
    if (u_willInvert > 0.5)
        gl_FragColor = vec4(alpha - outColor.r, alpha - outColor.g, alpha - outColor.b, alpha);
    else
        gl_FragColor = vec4(outColor.r, outColor.g, outColor.b, alpha);
}