#ifdef GL_ES
#define LOWP lowp
precision mediump float;
#else
#define LOWP
#endif

uniform sampler2D u_texture;
uniform LOWP vec3 u_tintRGB;
uniform float u_tintAmount;
uniform float u_saturation;
uniform float u_willInvert;
uniform float u_hueAdjust;

varying vec2 v_texCoord;
varying vec2 v_blurTexCoords[4];
varying vec4 v_blurTexCoordsPacked[2];
varying LOWP vec4 v_color;
 
const mat3 rgb2yiq = mat3(0.299, 0.595716, 0.211456, 0.587, -0.274453, -0.522591, 0.114, -0.321263, 0.311135);
const mat3 yiq2rgb = mat3(1.0, 1.0, 1.0, 0.9563, -0.2721, -1.1070, 0.6210, -0.6474, 1.7046);

void main()
{
   vec4 summedColor = vec4(0.0);
   float overallAlpha = 0.0;
   int i = 0;
   vec4 fetchedColor;
   float scaleWithAlpha;

    // 0

    fetchedColor = texture2D(u_texture, v_texCoord);
    float factor = 1.0 - fetchedColor.a * 0.8;

    if (fetchedColor.a > 0.0) {
        fetchedColor.rgb /= fetchedColor.a;      // ADDED this to get unpremultiplied rgb, then used that with premultiply opengl blend mode, work
        scaleWithAlpha = fetchedColor.a * 0.8;
        summedColor += fetchedColor * scaleWithAlpha;
        overallAlpha += scaleWithAlpha;
    }

    // 1

    fetchedColor = texture2D(u_texture, v_blurTexCoords[0]);
    if (fetchedColor.a > 0.0) {
        fetchedColor.rgb /= fetchedColor.a;
        scaleWithAlpha = fetchedColor.a * 0.21755 * factor;
        summedColor += fetchedColor * scaleWithAlpha;
        overallAlpha += scaleWithAlpha;
    }

    fetchedColor = texture2D(u_texture, v_blurTexCoords[1]);
    if (fetchedColor.a > 0.0) {
        fetchedColor.rgb /= fetchedColor.a;
        scaleWithAlpha = fetchedColor.a * 0.21755 * factor;
        summedColor += fetchedColor * scaleWithAlpha;
        overallAlpha += scaleWithAlpha;
    }

    // 2

    fetchedColor = texture2D(u_texture, v_blurTexCoords[2]);
    if (fetchedColor.a > 0.0) {
        fetchedColor.rgb /= fetchedColor.a;
        scaleWithAlpha = fetchedColor.a * 0.14915 * factor;
        summedColor += fetchedColor * scaleWithAlpha;
        overallAlpha += scaleWithAlpha;
    }

    fetchedColor = texture2D(u_texture, v_blurTexCoords[3]);
    if (fetchedColor.a > 0.0) {
        fetchedColor.rgb /= fetchedColor.a;
        scaleWithAlpha = fetchedColor.a * 0.14915 * factor;
        summedColor += fetchedColor * scaleWithAlpha;
        overallAlpha += scaleWithAlpha;
    }

    // 3 (NOW USING PACKED UV VARYING)

    fetchedColor = texture2D(u_texture, v_blurTexCoordsPacked[0].xy);
    if (fetchedColor.a > 0.0) {
        fetchedColor.rgb /= fetchedColor.a;
        scaleWithAlpha = fetchedColor.a * 0.08535 * factor;
        summedColor += fetchedColor * scaleWithAlpha;
        overallAlpha += scaleWithAlpha;
    }

    fetchedColor = texture2D(u_texture, v_blurTexCoordsPacked[0].zw);
    if (fetchedColor.a > 0.0) {
        fetchedColor.rgb /= fetchedColor.a;
        scaleWithAlpha = fetchedColor.a * 0.08535 * factor;
        summedColor += fetchedColor * scaleWithAlpha;
        overallAlpha += scaleWithAlpha;
    }

    // 4

    fetchedColor = texture2D(u_texture, v_blurTexCoordsPacked[1].xy);
    if (fetchedColor.a > 0.0) {
        fetchedColor.rgb /= fetchedColor.a;
        scaleWithAlpha = fetchedColor.a * 0.04795 * factor;
        summedColor += fetchedColor * scaleWithAlpha;
        overallAlpha += scaleWithAlpha;
    }

    fetchedColor = texture2D(u_texture, v_blurTexCoordsPacked[1].zw);
    if (fetchedColor.a > 0.0) {
        fetchedColor.rgb /= fetchedColor.a;
        scaleWithAlpha = fetchedColor.a * 0.04795 * factor;
        summedColor += fetchedColor * scaleWithAlpha;
        overallAlpha += scaleWithAlpha;
    }

    overallAlpha *= v_color.a;

    if (overallAlpha <= 0.0)
        discard;
    else
    {
        summedColor *= v_color.a;

		// Hue shift
		if (u_hueAdjust > 0.0 && u_hueAdjust < 1.0) {
			vec3 yColor = rgb2yiq * summedColor.rgb;
			
			//float originalHue = atan(yColor.b, yColor.g);
			// So atan(x,y) doesn't play nice if x==0.0, somehow it is for black/white and generally on IOS it didn't work - read the functions for "gotchas" if future glsl issues
			float originalHue = (yColor.g == 0.0 ? sign(yColor.b)*1.570795 : atan(yColor.b, yColor.g));
			float finalHue = originalHue + u_hueAdjust * 6.28318;
			
			float chroma = sqrt(yColor.b * yColor.b + yColor.g * yColor.g);
			
			vec3 yFinalColor = vec3(yColor.r, chroma * cos(finalHue), chroma * sin(finalHue));
			summedColor.rgb = yiq2rgb * yFinalColor;
		}

        // Tint
        summedColor.rgb = mix(summedColor.rgb, u_tintRGB * overallAlpha, u_tintAmount);

        // Saturate
        vec3 w = vec3(0.2125, 0.7154, 0.0721);
        vec3 intensity = vec3(dot(summedColor.rgb, w));
		summedColor.rgb = mix(summedColor.rgb, intensity, u_saturation + 1.0);
		
        // Invert
        if (u_willInvert > 0.5)
	        gl_FragColor = vec4(overallAlpha - summedColor.r, overallAlpha - summedColor.g, overallAlpha - summedColor.b, overallAlpha);
	    else
	        gl_FragColor = vec4(summedColor.r, summedColor.g, summedColor.b, overallAlpha);
    }
}