#ifdef GL_ES
#define LOWP lowp
precision mediump float;
#else
#define LOWP
#endif

uniform sampler2D u_texture;
uniform vec2 u_fboSize;
uniform float u_is_vertical;
uniform float u_blur_intensity;

varying vec2 v_texCoord;
varying LOWP vec4 v_color;

float SCurve (float x) {
    x = x * 2.0 - 1.0;
    return -x * abs(x) * 0.5 + x + 0.5;
    //return dot(vec3(-x, 2.0, 1.0 ),vec3(abs(x), x, 1.0)) * 0.5; // possibly faster version
}

vec4 BlurH (sampler2D source, vec2 size, vec2 uv, float radius) {
    if (radius >= 1.0)
    {
        vec4 A = vec4(0.0);
        vec4 C = vec4(0.0);

        float width = 1.0 / size.x;

        float divisor = 0.0;
        float weight = 0.0;

        float radiusMultiplier = 1.0 / radius;

        // Hardcoded for radius 20 (normally we input the radius
        // in there), needs to be literal here

        for (float x = -radius; x <= radius; x++)
        {
            A = texture2D(source, uv + vec2(x * width, 0.0));

            weight = SCurve(1.0 - (abs(x) * radiusMultiplier));

            C += A * weight;

            divisor += weight;
        }

        return vec4(C.r / divisor, C.g / divisor, C.b / divisor, C.a / divisor);
    }

    return texture2D(source, uv);
}

vec4 BlurV (sampler2D source, vec2 size, vec2 uv, float radius) {
    if (radius >= 1.0)
    {
        vec4 A = vec4(0.0);
        vec4 C = vec4(0.0);

        float height = 1.0 / size.y;

        float divisor = 0.0;
        float weight = 0.0;

        float radiusMultiplier = 1.0 / radius;

        for (float y = -radius; y <= radius; y++)
        {
            A = texture2D(source, uv + vec2(0.0, y * height));

            weight = SCurve(1.0 - (abs(y) * radiusMultiplier));

            C += A * weight;

            divisor += weight;
        }

        return vec4(C.r / divisor, C.g / divisor, C.b / divisor, C.a / divisor);
    }

    return texture2D(source, uv);
}

void main()
{
    // Apply vertical blur to buffer A
    if (u_is_vertical > 0.0)
    gl_FragColor = BlurV(u_texture, u_fboSize.xy, v_texCoord, u_blur_intensity * 20.0);
    else
    gl_FragColor = BlurH(u_texture, u_fboSize.xy, v_texCoord, u_blur_intensity * 20.0);
}