uniform mat4 u_projTrans;
uniform float u_blurPixels;
uniform float u_texelSize;

attribute vec4 a_position;
attribute vec2 a_texCoord0;
attribute vec4 a_color;

varying vec2 v_texCoord;
varying vec2 v_blurTexCoords[4];
varying vec4 v_blurTexCoordsPacked[2];  // need to pack the remaining ones, max of 8 varyings allowed with some devices, will use all xyzw twice for 4 more UVs, perf hit? so only using min needed packed UVs
varying vec4 v_color;
 
void main()
{
    v_texCoord = a_texCoord0;
    v_color =  a_color;
	v_color.a = v_color.a * (255.0 / 253.0);    // default libgdx shaders are 255/254, but this didn't work here, so I made it 253
    gl_Position = u_projTrans * a_position;

    float blurDistance3 = u_blurPixels * u_texelSize;
    float blurDistance2 = blurDistance3 * 0.75;
    float blurDistance1 = blurDistance3 * 0.5;
    float blurDistance0 = blurDistance3 * 0.25;

    v_blurTexCoords[0] = v_texCoord + vec2(-blurDistance0, 0.0);
    v_blurTexCoords[1] = v_texCoord + vec2( blurDistance0, 0.0);
    v_blurTexCoords[2] = v_texCoord + vec2(-blurDistance1, 0.0);
    v_blurTexCoords[3] = v_texCoord + vec2( blurDistance1, 0.0);
    v_blurTexCoordsPacked[0].xy = v_texCoord + vec2(-blurDistance2, 0.0);
    v_blurTexCoordsPacked[0].zw = v_texCoord + vec2( blurDistance2, 0.0);
    v_blurTexCoordsPacked[1].xy = v_texCoord + vec2(-blurDistance3, 0.0);
    v_blurTexCoordsPacked[1].zw = v_texCoord + vec2( blurDistance3, 0.0);
}